@@ -1,4 +1,4 @@
-# -*- Mode: Python; py-indent-offset: 4 -*-
+# -*- Mode: Python -*-
 # pygobject - Python bindings for the GObject library
 # Copyright (C) 2006  Johannes Hoelzl
 #
@@ -164,8 +164,9 @@ class OptionGroup(optparse.OptionGroup):
 
         self.translation_domain = translation_domain
 
-        for option in option_list:
-            self.add_option(option)
+        if option_list:
+            for option in option_list:
+                self.add_option(option)
 
     def _create_option_list(self):
         self.option_list = []
@@ -312,11 +313,17 @@ class OptionParser(optparse.OptionParser):
 
     def _process_args(self, largs, rargs, values):
         context = self._to_goptioncontext(values)
-        largs.extend(context.parse([sys.argv[0]] + rargs))
+        
+        # _process_args() returns the remaining parameters in rargs.
+        # The prepended program name is used to all g_set_prgname()
+        # The program name is cut away so it doesn't appear in the result.
+        rargs[:] = context.parse([sys.argv[0]] + rargs)[1:]
 
     def parse_args(self, args=None, values=None):
+        old_args = args or []
         try:
-            return optparse.OptionParser.parse_args(self, args, values)
+            options, args = optparse.OptionParser.parse_args(
+                self, args, values)
         except gobject.GError, error:
             if error.domain != gobject.OPTION_ERROR:
             	raise
@@ -329,4 +336,11 @@ class OptionParser(optparse.OptionParser):
             else:
                 raise
 
+        for group in self.option_groups:
+            for key, value in group.values.__dict__.items():
+                options.ensure_value(key, value)
+
+        args = args[2:-len(old_args)]
+	return options, args
+
 make_option = Option
